/* Multipattern random searches using set Horspool's algorithm adapted to 
search inside a text compressed with (S,C)-Dense Code.
Lightweight Natural Language Text Compression: Information Retrieval 2006
Copyright (C) 2005 Antonio Faria.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author's contact: Antonio Faria, Dept. of Computer Science, University of
A Corua. Campus de Elvia s/n. Spain  fari@udc.es
*/

/*
BufferedReader is a modification of the buffering routines found in
"Nrgrep -- a fast and flexible pattern matching tool.
Copyright (C) 2000 Gonzalo Navarro"
http://www.dcc.uchile.cl/~gnavarro/software/sindex.html
*/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <string.h>
#include <fcntl.h>

#include "constants.h"

#ifndef BUFFERINCLUDED
#define BUFFERINCLUDED

#define BUFFER_SIZE 1024*1024

	/* Buffer management */

typedef struct sBuffer
  { int size;		/* size of this buffer */
    int file;		/* associated file */
    int fpos;		/* pos of current buffer in file */
    byte *data;	/* current text */
    int dsize;		/* size of current text */
    int minDataInBuffer;
  } *BufferRead;

	/* Creates a new buffer */

BufferRead bufCreate (unsigned int minDataInBuffer);

	/* Terminates use of buffer B */

void bufDestroy (BufferRead B);

	/* Assigns a file to B */

void bufSetFile (BufferRead B, int file);

        /* Gets buffer size of B */                                             

int bufSizer (BufferRead B);
 
	/* Reads a new buffer starting at position pnext */

void bufLoad (BufferRead B, byte *pnext);

  	/* Tells whether there is no more data in B + its file */

int bufEmpty (BufferRead B);

  	/* Tells whether there is no more data to read in B's file */

int bufEof (BufferRead B);

	/* Tells the file position of the current buffer */

int bufTextPos (BufferRead B);

	/* Gives the memory area of the current buffer */
	/* One text position before the given buffer can be touched */

void bufCurrent (BufferRead B, byte **base, byte **top);

#endif


//
///*-----------------------------------------------------------------------
// Arquivo    : Set horspool modified.h
// Function   : Implments a BufferReader keeping at least the "minDataInBuffer"
//  			  previous bytes read available to go back in the buffer read.
// Last change: 19/01/2005
// ------------------------------------------------------------------------*/
//
//
//#include <stdlib.h>
//#include <stdio.h>
//#include <unistd.h>
//
//#include <string.h>
//#include <fcntl.h>
//
//#include "constants.h"
//
////#define BUFFER_SIZE 1024*1024*4
//#define BUFFER_SIZE 1024*1024*4
////#define BUFFER_SIZE 20
//
//	/* Buffer management */
//
//typedef struct sBuffer
//  { int size;		/* size of this buffer */
//    int file;		/* associated file */
//    int fpos;		/* pos of current buffer in file */
//    byte *data;	/* current text */
//    int dsize;		/* size of current text */
//    int minDataInBuffer;  		/*except for the first buffer, ensures that number of bytes before "current pos" */
//    int minDataInBufferRight; /*except for the last buffer, that number of bytes will be always "readable" at the end of the buffer */
//  } *BufferRead;
//
//	/* Creates a new buffer */
//
//BufferRead bufCreate (unsigned int minDataInBuffer,unsigned int minDataInBufferRight);
//
//	/* Terminates use of buffer B */
//
//void bufDestroy (BufferRead B);
//
//	/* Assigns a file to B */
//
//void bufSetFile (BufferRead B, int file);
//
//        /* Gets buffer size of B */
//
//int bufSizer (BufferRead B);
//
//	/* Reads a new buffer starting at position pnext */
//
//void bufLoad (BufferRead B, byte *pnext);
//
//  	/* Tells whether there is no more data in B + its file */
//
//int bufEmpty (BufferRead B);
//
//  	/* Tells whether there is no more data to read in B's file */
//
//int bufEof (BufferRead B);
//
//	/* Tells the file position of the current buffer */
//
//int bufTextPos (BufferRead B);
//
//	/* Gives the memory area of the current buffer */
//	/* One text position before the given buffer can be touched */
//	/* The data will be in [base, top) */
//
//void bufCurrent (BufferRead B, byte **base, byte **top);
//
//	/* Gives a pointer to the end of the memory area reserved for data in buffer */
//	/* Exactly to one position further of the end of the DATA - AREA */
//	/* 1 + BUFFER_SIZE   from a full size of (1 + BUFFER_SIZE +4) */
//void bufEndData (BufferRead B, byte **base);
//
