/* Multipattern random searches using set Horspool's algorithm adapted to 
search inside a text compressed with (s,c)-Dense Code.
Lightweight Natural Language Text Compression: Information Retrieval 2006
Copyright (C) 2005 Antonio Faria.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author's contact: Antonio Faria, Dept. of Computer Science, University of
A Corua. Campus de Elvia s/n. Spain  fari@udc.es
*/

/*-----------------------------------------------------------------------
  RandomSearch.c:     Main file of the multipattern-searcher.
  Loads the vocabulary of the compressor into a hash table. Then it loads a set of
  patterns (one set for each iteration of the search process). Such set of patterns
  is searched inside the compressed file.  
 ------------------------------------------------------------------------*/
 
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdarg.h>
#include <stdlib.h>

#include <stdarg.h>  //para el vscanf

#include "tiempo.h"
#include "hash.h"
#include "searcher.h"

struct Vocab {
        unsigned char   *palabra;   // palabra del vocabulario
        unsigned long 	numBytesPalabra;
        unsigned long 	frecuencia;
} ;

typedef struct Vocab t_Vocab;

t_Vocab *lista;

FILE *ftiempos;
FILE *fPatterns;  //file containing the patterns.


/*-----------------------------------------------------------------------
 Loads the vocabulary used by the decompressor into a hash table. The key of the hash table 
 is the word,  and the value --> the relative position in the  ordered vocabulary.
 ------------------------------------------------------------------------*/
void loadCompressorHashTable(char * fileName) {
	BufferRead V;	
	int v;
	unsigned char *text,*top, *nextbuf, *pbeg, *pend, *codeStart;
	char *voc;

	voc = (char *) malloc(strlen(fileName)+4);
	strcpy(voc,fileName);
	strcat(voc,".voc");	
	
	// Inicialization of the buffers used to read the files

	v = open (voc, O_RDONLY);
	if (v == -1) {
		fprintf(stderr,"Cannot read vocabulary file %s\nSearch aborted.\n",voc);
		exit(0);
	}	

	V = bufCreate(MAX_SIZE_OF_WORD+5);	  //a word being "cut"
	bufSetFile (V,v);
	
	/* loading the vocabulary into memory*/
	bufCurrent (V,(byte**)&text,(byte**)&top);
	pbeg = text; pend = top;	
	  		
	{ //Reads the number of words of the vocabulary (stored in the text file as a long value)
		unsigned long numbertmp;
		int z;
		unsigned char *c;
		c= (unsigned char*) &numbertmp;
		for (z=1; z<=4;z++) {
			*c =*pbeg;
			c++;
			pbeg++;
		}
		fprintf(stderr,"\n\t*Number of words in the vocabulary is: %ld",numbertmp);
	
		initialize_hash(numbertmp); //initializes decompressor for up to N_value words
	}			
	
	//		setSValue(128);
	{//Reads the value of S (and C = 256-S)
		byte valor_S[4];
		int S;
		codeStart = pbeg;
		while (*pbeg !='\0') {
			pbeg++;
		}

		{
			register unsigned long len;
			register int i;
			len = pbeg-codeStart;
			for (i=0;i<=len;i++) {  //copia tambien el '\0'
				valor_S[i]=*(codeStart++);
			}
		}
		
		S=atoi(valor_S);
		setSValue(S);
		fprintf(stderr,"\n\t*Value of S is: %ld, C is: %ld",S_value,C_value);		
		pbeg++;
	}
		
	/*Reads one word at a time (j) and adds it to the vocabulary vector.*/
	{	unsigned long j;
		unsigned long RELPOS = 0;
		j=0;
		while(1){
		
			while ( pbeg < pend )   {
				//reads the word
				codeStart = pbeg;
				while (*pbeg !='\0') {
					pbeg++;
				}
				
				//adds the word to the vocabulary
				{	
					unsigned long addr;
	
					register unsigned long len;
					unsigned char *w;
					register int i;
			
					len = pbeg-codeStart;
					j = search ((unsigned char *)codeStart, len, &addr);			
											
					//adds the word to the hash Table

					insertElement( codeStart, len,&addr);
					//						w=hash[addr].word;
					//						for (i=0;i<=len;i++) {  //copia tambien el '\0'
					//							*(w++)=*(codeStart++);
					//						}

					hash[addr].len=len;
					{	unsigned long aCode;
						getCode(RELPOS,&(aCode), &(hash[addr].size));
						
						getBytes(hash[addr].codeword, aCode, hash[addr].size);						
						//hash[addr].codeword = ;
						//hash[addr].len= ;
						
						//fprintf(stderr,"\n %ld",RELPOS);
						//printCodigo (hash[addr].codeword, hash[addr].size);
					}
					RELPOS ++;
				}
			
				j++;
				pbeg++;
			}
			
		nextbuf=pbeg;
					
		bufLoad (V, (char*)nextbuf);
		if (bufEmpty(V)) break;
		bufCurrent (V,(byte**)&text,(byte**)&top);
		pbeg = text; pend = top;
		}
	}
	
	bufDestroy(V);
	close(v);
	free(voc);		
}

	
/*-----------------------------------------------------------------------
 Loads the next "numpatterns from the file of patterns". 
 ------------------------------------------------------------------------*/
void CargaVocabulario(FILE *file, unsigned long numPats){

    unsigned long int i;
    unsigned long int numBytesPalabra;
    unsigned long int frecuencia;
    unsigned char palabra[1024];

	//FORMAT OF THE FILE ...
	// [OCCUR] [SIZE] [WORD]
	//  %9ld   %9ld   %s\n"

	lista= (t_Vocab *) malloc(numPats * sizeof(t_Vocab));

	for (i=0;i<numPats;i++) {
		fscanf(file,"%9ld ",&frecuencia);
		fscanf(file,"%9ld ",&numBytesPalabra);

		lista[i].palabra = (unsigned char *) malloc (numBytesPalabra +1);
		fscanf(file,"%s\n", lista[i].palabra);
		lista[i].palabra[numBytesPalabra]='\0';

		lista[i].frecuencia = frecuencia;
		lista[i].numBytesPalabra = numBytesPalabra;
	}
}



//*********************************************************************
//*********************************************************************
//*********************************************************************

int main (int argc, char *argv[]) {
	unsigned int z;
	unsigned char nombreFichTiempos[100];

	unsigned int iterations = 1;
	unsigned int iterActual =1;
	unsigned int numPatterns =1;

	#ifdef LOGGING_ENABLED
		FILE *fLOG1;
	#endif

	unsigned long pos,i;
	
	tTiempo miTiempo1;
	tTiempo miTiempo2;

	tTiempo miTiempo11;
	tTiempo miTiempo12;

	unsigned long *chosenPos;
	byte **patterns;   //patterns to be searched for (in each iteration)
	int *lens;   	   //len of the patterns searched in each iteration

	double totalTime;
	double totalVarianza;
  	totalTime =0.0;
  	totalVarianza = 0.0;

  	
	if (argc < 7) {
		printf("\nUsage: %s <compressedFile> <patsFile> <iterations> <FTIMES> <numPatternsPerIter> <strmatlab>\n\n", *argv);
		exit(0);
	}

	//prepares the file that will contain the measured times.
	nombreFichTiempos[0]='\0';
	strcat(nombreFichTiempos,argv[4]);
	strcat(nombreFichTiempos,"_.m");
	if ((ftiempos = fopen(nombreFichTiempos,"w")) == NULL)	{
		  fprintf(stderr,
			  "Error opening file for times: %s\n", nombreFichTiempos);
		  exit(0);
	}

	getTiempo(&miTiempo1);
	getTiempo(&miTiempo2);

	//process parameters
	iterations = atoi(argv[3]);
	numPatterns = atoi(argv[5]);

	fprintf(stderr, "fichero de tiempos %s\n", nombreFichTiempos);
	fprintf(stderr, "fichero comprimido: [%s],  fichero vocabulario: [%s]\n", argv[1], argv[2]);
	fprintf(stderr, "iterations [%d], numPatterns [%d]\n", iterations, numPatterns);

	_memMgr = createMemoryManager();

	//opens the file containing the random patterns
	if ((fPatterns = fopen(argv[2],"r")) == NULL)
	{
	  fprintf(stderr,"\nError en la apertura del fichero %s\n", argv[2]);
	  exit(0);
	}
	
		
	//loads the vocabulary of the compressor into a hash table that will be used to obtain
	//the codewords associated to a search pattern (a word), previously to perform a search.
	loadCompressorHashTable(argv[1]);

	//allocates memory
	patterns =(byte **) malloc(1*4*numPatterns);  //allocates 4 bytes per pattern
	lens =(int *) malloc(1*4*numPatterns);
	chosenPos= (unsigned long *) malloc(1*4*numPatterns);

	//Searches for "numPatterns" random patterns, "iterations" times.
	//To do that, "numPatterns" are chosen from the available patterns  
	//during the call to CargaVocabulario(), which loads them into "lista".

	//FOR LOOP OF ITERATIONS STARTS. ******************
	getTiempo(&miTiempo11);
	getTiempo(&miTiempo1);

	// iterations ... *****************************************************
	for (;iterActual <= iterations; iterActual++) {

		#ifdef LOGGING_ENABLED
			if ((fLOG1 = fopen("log1","w")) == NULL) {
				fprintf(stderr,"Error en la apertura del fichero de tiempos \n");
				exit(0);
			}
		#endif
		
		fprintf(stderr,"\n Search: %d of %d",iterActual,iterations);
		
		//Read "numPatterns" random patterns...
		fprintf(stderr, "\n READING RANDOM PATTERNS... \n");
		
		CargaVocabulario(fPatterns,numPatterns);   //loads the next "numPatterns" patterns from the "patterns file"
		
		#ifdef LOGGING_ENABLED
			for (i=0;i<numPatterns;i++) {
				//writes the pattern to the file LOG1
				fprintf(fLOG1, "%5d.- %6d\n",i,lista[i].frecuencia);
			}
		#endif
		
				
		//Once the random compressed patterns are known, we search for them into the file.
		//  Time measures take only into account the time needed to search for each pattern inside the hash table
		//  of patterns, and the traversing the full file searching for the compressed search patterns. 
		//  It skips times needed to load the vocabulary, the hash table, ...
		/*********************/
		getTiempo(&miTiempo1);
		
		{//Searches for the loaded patterns inside the hash table 
		 //	and obtains their codeword and codeword lenght.
			unsigned long addr;
			
			//fills patterns[] and lens[] vectors.
			for (i=0;i<numPatterns;i++) {

				pos=i;
				z=search (lista[pos].palabra, lista[pos].numBytesPalabra, &addr);	
				patterns[i]= (byte *) hash[addr].codeword;
				lens[i] = hash[addr].size;			
				
				#ifdef LOGGING_ENABLED
					fprintf(stderr, "\n%d . [%s[%d]] ",i,lista[pos].palabra,lista[pos].frecuencia);
					printCodigo (patterns[i], lens[i]);
				#endif				
			}
		}

		//searches inside the compressed file.
		searchFullFile(argv[1],numPatterns,patterns,lens);

		getTiempo(&miTiempo2);
		/*********************/
		
		totalTime  += miTiempo2.user - miTiempo1.user;	
		totalVarianza += ((miTiempo2.user - miTiempo1.user) * (miTiempo2.user - miTiempo1.user));
		
		fprintf(ftiempos,"%s(%d) = %f ;\n", argv[6],(iterActual) , miTiempo2.user - miTiempo1.user);
		fflush(ftiempos);
		muestraTiempo(miTiempo1,miTiempo2,"SEARCH TIME");

		#ifdef LOGGING_ENABLED

			fflush(fLOG1);
			fclose(fLOG1);
		#endif
	
		//frees memory.
		for (i=0;i<numPatterns;i++) {free (lista[i].palabra);}
		free(lista);
		
	} //end for iterations. *****************************************************

	getTiempo(&miTiempo12);

	
	//Gives some statistics of the current run...
	fprintf(stderr,"\n AVG USER TIME (only search process) = %f sec.",totalTime/iterations);
	{	double varr;		
		varr= (totalVarianza / iterations) - ((totalTime / iterations) *(totalTime / iterations));
		fprintf(stderr,"\n Desv est : %f  ", sqrt(varr * iterations / (iterations -1))  );
		fprintf(stderr,"\n Varianza : %f  ", varr  );
		fprintf(stderr,"\n Desv tp : %f  \n\n", sqrt(varr)  );
	}	
			
	fprintf(ftiempos,"mean(%s)\n", argv[6] );
	fprintf(ftiempos,"std(%s)\n", argv[6] );
	fprintf(ftiempos,"%s=sort(%s);\n", argv[6],argv[6]  );
	fprintf(ftiempos,"plot(%s,'b')\n", argv[6]);
	
	//Frees memory, closes file handlers ...........
	fclose (ftiempos);
	fclose(fPatterns);

	free(lens);
	free(chosenPos);
	
	free(patterns);
	freeHashTable();
}
