/* Multipattern random searches using set Horspool's algorithm adapted to 
search inside a text compressed with SC-Dense Code.
Lightweight Natural Language Text Compression: Information Retrieval 2006
Copyright (C) 2005 Antonio Faria.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author's contact: Antonio Faria, Dept. of Computer Science, University of
A Corua. Campus de Elvia s/n. Spain  fari@udc.es
*/

/*-----------------------------------------------------------------------
  search.c:     Implements a trie and deals with the search of a set of patterns
  inside a compressed file (searchFullFile()). Searching the whole file is done 
  one buffer at a time.
 ------------------------------------------------------------------------*/
 
#ifndef SEARCHERINCLUDED
#define SEARCHERINCLUDED
#endif

#include <string.h>
#include <stdlib.h>
#include <math.h>

#include <malloc.h>
#include <stdio.h>

#include "BufferedReader.h"
#include "stack.h"




#define LINE_UP(x,numBytes) ((x>>(4-numBytes)*8))    //move "4-bytes" bytes to the right
						   						   			  //so we can write "bytes" bytes from &x
#ifdef LITTLE_ENDIAN
	#define LE_LONG32(x)  (((x&0xff000000)>>24)|((x&0x00ff0000)>>8)|((x&0x0000ff00)<<8)|((x&0x000000ff)<<24))
#else
	#define LE_LONG32(x) x
#endif


int S_value;
int C_value;


#define MIN(a,b) (a < b) ? a : b
#define MAX(a,b) (a > b) ? a : b


//#define MAX_STATES 2048
//#define MAX_STATES 4096
//#define NORMAL 4096         //It must be an integer int smaller than "NUMBER OF PATTERNS"

#define MAX_STATES 4096
#define NORMAL 8192         //It must be an integer int smaller than "NUMBER OF PATTERNS"

//#define MAX_STATES 16384
//#define NORMAL 16385

#define ROOT_STATE  0

#define INFINITE 100000

	//stores information about each search pattern
	struct t_patt {
		byte word[MAX_SIZE_OF_WORD+MAX_CODEWORD_SIZE+2];
		unsigned int size;
		unsigned long matches;
		unsigned int flag;
	};
	typedef struct t_patt t_pattern;

	struct t_pats {        //a structure containing the list of searched patterns
		t_pattern *patterns;
		unsigned int size;
		unsigned int lmin;
		unsigned int lmax;
	};
	typedef struct t_pats *t_patterns;


	//definition of a trie

	struct tnod {
		unsigned int n;  //number of patterns this node occurs in
		unsigned int f;  //type of node (and reference to the vector of patterns		
	};
	typedef struct tnod tnodetrie;

	struct t_ttrie {
		tnodetrie nodes[MAX_STATES];         			//nodes of the trie
		unsigned int transitions[MAX_STATES][256];   //transitions for each node of the tree.
		unsigned long zeroNode;
		t_stack stack;
	};
	typedef struct t_ttrie *t_trie ;                //Our "trie" data type


#ifdef CONTADOR_BYTES_ENABLED
	long LEIDOS;
#endif

#ifdef LOGGING_ENABLED
	FILE *fLOG2;
#endif 


/*********************************************/
 t_patterns initializePatterns(int numpatts, byte **patts, int *lens);
 void inicialize_d(t_patterns P, byte *d);
 void updateDVector(int newSize, byte *oldPattern, int oldSize, t_patterns P, byte *d);
 void updateDVectorByAddition(int newSize, t_patterns P, byte *d);
 t_trie inicializeTrie(t_patterns P);
 void printTrie(t_trie trie,int maxstate);
 void removeFromTrie(t_trie trie, byte *code, int size);
 void addToTrie(t_trie trie, byte *nuevo, int size, unsigned int flag);
 void searchInBufferETDC (BufferRead B, byte **begin, byte **end,unsigned long pos,
					 t_patterns P, t_trie trie, byte *d );
 
 void searchFullFile (char *infile, int numWords, byte **words, int *lens);

