/* Multipattern random searches using set Horspool's algorithm adapted to 
search inside a text compressed with SC-Dense Code.
Lightweight Natural Language Text Compression: Information Retrieval 2006
Copyright (C) 2005 Antonio Faria.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author's contact: Antonio Faria, Dept. of Computer Science, University of
A Corua. Campus de Elvia s/n. Spain  fari@udc.es
*/

/*-----------------------------------------------------------------------
 stack.c: Definition of a stack (statically defined) data structure.
	It is used by the trie of the searcher
 ------------------------------------------------------------------------*/
 
#include "stack.h"

/*------------------------------------------------------------------
 Constructor method
 ------------------------------------------------------------------ */
t_stack createStack(unsigned int size) {
	t_stack stack;
	stack = (t_stack) malloc (sizeof(struct sstack));
	stack->top=-1;
	stack->size = size;
	stack->data = (unsigned int *) malloc (sizeof(unsigned int) * size);
	return stack;
}

/*------------------------------------------------------------------
 Destructor method
 ------------------------------------------------------------------ */
void destroyStack (t_stack stack){
	free(stack->data);
}

/*------------------------------------------------------------------
 Gets the top of the stack
 ------------------------------------------------------------------ */
unsigned int pop(t_stack stack){
	int top = stack->top;
	if (! isEmpty(stack)) {
		stack->top --;
		return stack->data[top];
	}
	else {
		fprintf(stderr,"\n THE STACK IS EMPTY\n ");
		exit(0);	
	}
}


/*------------------------------------------------------------------
 Adds a new alement to the stack
 ------------------------------------------------------------------ */
void push (t_stack stack, unsigned int element){
//	fprintf(stderr,"\n pushing[%d]",element);
	if (! isFull(stack)) {
		stack->top ++;
		stack->data[stack->top] = element;
	}
//	fprintf(stderr,"\n out pushing[%d]",stack->data[stack->top]);	
}

/*------------------------------------------------------------------
 Tells if the stack is empty: //RETURNs 1 if it is empty
 ------------------------------------------------------------------ */
int isEmpty(t_stack stack){
	return (stack->top == -1);	
}

/*------------------------------------------------------------------
 Tells if the stack is full: //RETURNs 1 if it is full
 ------------------------------------------------------------------ */
int isFull(t_stack stack){
	return (stack->top == stack->size-1);	
}

/*------------------------------------------------------------------
 Shows the data in the stack from bottom to top.
 ------------------------------------------------------------------ */
void printStack(t_stack stack){
	int i;
	fprintf(stderr,"\n\n DATA IN THE STACK \n*******************");
	if (isEmpty(stack)) {
			fprintf(stderr,"\n THE STACK IS EMPTY\n ");
			return;
	}
	for (i =0; i<= stack->top; i++) {
		fprintf(stderr,"\n stack[%d] -> %d ",i, stack->data[i]);
	}
}

/*------------------------------------------------------------------
 main, to make unit proofs
 ------------------------------------------------------------------ */
/*
int main(int argc, char* argv[])
{  t_stack stack
	
		stack = createStack(256);

			{ int tmp;
			printStack(stack);
			push(stack,100);
			push(stack,101);
			push(stack,102);
			push(stack,103);
			push(stack,104);
			printStack(stack);	
				while (!isEmpty(stack)) {
					push(stack,tmp);
					tmp = pop(stack);
					fprintf(stderr,"\n poped element [%d]",tmp);
				
				}
			}
			

}
*/
