/* Multipattern random searches using set Horspool's algorithm adapted to 
search inside a text compressed with SC-Dense Code.
Lightweight Natural Language Text Compression: Information Retrieval 2006
Copyright (C) 2005 Antonio Faria.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author's contact: Antonio Faria, Dept. of Computer Science, University of
A Corua. Campus de Elvia s/n. Spain  fari@udc.es
*/

/* tiempo.c: Gives support to take time measures 
	Using either getTiempo we take note of the current moment of "time"
	Differences between two moments of time permit to obtain execution time.
*/

#include "tiempo.h"

/*------------------------------------------------------------------
 By using gettimeofday and getrusage, it obtains current time.
 ------------------------------------------------------------------ */
void getTiempo(tTiempo *tiempo)
{
	double mega = 1.0e-6;
	struct rusage buffer;
	struct timeval tp;
	struct timezone tzp;

	getrusage(RUSAGE_SELF, &buffer);
	gettimeofday(&tp, &tzp);
//	tiempo->user = (double) buffer.ru_utime.tv_sec * 1000000 +
//				     buffer.ru_utime.tv_usec;

	tiempo->user = (double) buffer.ru_utime.tv_sec +
				    1.0e-6 * buffer.ru_utime.tv_usec;
//	tiempo->sys   = (double) buffer.ru_stime.tv_sec +
//				    1.0e-6 * buffer.ru_stime.tv_usec;
	tiempo->total = (double) tp.tv_sec + 1.0e-6 * tp.tv_usec;
}



/*------------------------------------------------------------------
 Obtains the difference in time between two moments (tTiempo) and displays it.
 ------------------------------------------------------------------ */
void muestraTiempo(tTiempo mitiempo1, tTiempo mitiempo2,char *mensaje)
{
	printf("   *******************************************\n");
	printf("   *%s  ",mensaje);
	// printf("   * real  %f\n",  mitiempo2.total - mitiempo1.total );
	printf("   * user  %f\n",  mitiempo2.user - mitiempo1.user );
	// printf("   * sys   %f\n",  mitiempo2.sys - mitiempo1.sys );
	// printf("   * user +sys   %f\n",  mitiempo2.sys - mitiempo1.sys + mitiempo2.user - mitiempo1.user);
	printf("   *******************************************");
	printf("\n ");

}






/*************************************************************************************************/
/** example of use .. **/
/*************************************************************************************************/
void main2() {
	double utime0, stime0, wtime0,
		   utime1, stime1, wtime1;
	register int i,j,k,a;

	tTiempo mitiempo1;
	tTiempo mitiempo2;




	/* ---- Primera invocacin a la funcin. ---- */
	getTiempo(&mitiempo1);

	/* ---- Segmento de cdigo cuyo tiempo de ejecucin se desea medir. --- */
	for(i=0; i<1000; i++)
	{	for(j=0; j<1000; j++)
			for(k=0; k<1000; k++)
			{a++;
			}
		printf("%d",i);

	}

	/* ---- Segunda invocacin a la funcin. ---- */
	//getTime(&mitiempo2.user, &mitiempo2.sys, &mitiempo2.total);
	getTiempo(&mitiempo2);

	/* --- Clculo del tiempo de ejecucin e impresin de resultados. --- */
	printf("\n");
	printf("real  %.3f\n",  mitiempo2.total - mitiempo1.total );
	printf("user  %.3f\n",  mitiempo2.user - mitiempo1.user );
	printf("sys   %.3f\n",  mitiempo2.sys - mitiempo1.sys );
	printf("\n");
	printf("\n");
}
